/*
 * Copyright 2005-2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.ldap.filter;

import com.gargoylesoftware.base.testing.EqualsTester;
import org.junit.Test;

import static org.junit.Assert.assertEquals;

/**
 * Unit tests for the HardcodedFilter class.
 *
 * @author Ulrik Sandberg
 */
public class HardcodedFilterTest {

    @Test
	public void testHardcodedFilter() {
		HardcodedFilter filter = new HardcodedFilter("(foo=a*b)");
		assertEquals("(foo=a*b)", filter.encode());

		NotFilter notFilter = new NotFilter(new HardcodedFilter("(foo=a*b)"));
		assertEquals("(!(foo=a*b))", notFilter.encode());

		AndFilter andFilter = new AndFilter();
		andFilter.and(new HardcodedFilter("(foo=a*b)"));
		andFilter.and(new HardcodedFilter("(bar=a*b)"));
		assertEquals("(&(foo=a*b)(bar=a*b))", andFilter.encode());

		andFilter = new AndFilter();
		andFilter.and(new HardcodedFilter("(foo=a*b)"));
		andFilter.and(new NotFilter(new HardcodedFilter("(bar=a*b)")));
		assertEquals("(&(foo=a*b)(!(bar=a*b)))", andFilter.encode());
	}

    @Test
    public void testEquals() {
		String attribute = "(foo=a*b)";
		HardcodedFilter originalObject = new HardcodedFilter(attribute);
		HardcodedFilter identicalObject = new HardcodedFilter(attribute);
		HardcodedFilter differentObject = new HardcodedFilter("(bar=a*b)");
		HardcodedFilter subclassObject = new HardcodedFilter(attribute) {
        };

        new EqualsTester(originalObject, identicalObject, differentObject,
                subclassObject);
    }
}